<?php

class woocommerce_feedoptimise_feed
{
    const VERSION = '1.1.13';

    protected $_custom_terms = '';
    protected $_terms;
    protected $_include_post_meta = false;

    function render()
    {
        //fix for websites that gzip the response but do not send the gzip header
        if(isset($_GET['woocommerce_fo_encoding']) && $_GET['woocommerce_fo_encoding'] != '')
        {
            header("Content-Encoding: ".$_GET['woocommerce_fo_encoding']);
        }
        header('Content-type: application/json');
        @header('X-Robots-Tag: "none, noindex, nofollow, noarchive, nosnippet, noodp, notranslate, noimageindex"');
        $this->_render_product_feed();
    }

    /**
     * Render the product feed requests - calls the sub-classes according
     * to the feed required.
     *
     * @access public
     */
    protected function _render_product_feed() {

        global $woocommerce, $wp_version;

        if(isset($_GET['woocommerce_fo_version']))
        {
            echo json_encode(
                array(
                    'foVersion'	            => self::VERSION,
                    'wooCommerceVersion'    => $woocommerce->version,
                    'wpVersion'             => @$wp_version
                )
            );

            exit;
        }

        if(isset($_GET['woocommerce_fo_enable_php_errors']))
        {
            error_reporting(E_ALL);
            ini_set("display_errors", 1);
        }
        elseif(isset($_GET['woocommerce_fo_disable_php_errors']))
        {
            error_reporting(0);
            ini_set("display_errors", 0);
        }

        global $wpdb, $wp_query, $post;

        // Don't cache feed under WP Super-Cache
        define('DONOTCACHEPAGE', TRUE);

        // Cater for large stores
        $wpdb->hide_errors();
        @set_time_limit(0);
        @ob_clean();

        // wp_suspend_cache_addition is buggy prior to 3.4
        if(version_compare(get_bloginfo('version'),'3.4','>=' ))
        {
            wp_suspend_cache_addition(true);
        }

        // Query for the products
        $chunk_size = (int)(isset($_GET['woocommerce_fo_chunk_size']) ? $_GET['woocommerce_fo_chunk_size'] : 20);
        $limit 		= (int)(isset($_GET['woocommerce_fo_limit']) ? $_GET['woocommerce_fo_limit'] : 0);
        $offset 	= (int)(isset($_GET['woocommerce_fo_offset']) ? $_GET['woocommerce_fo_offset'] : 0);

        $this->_custom_terms 	    = (isset($_GET['woocommerce_custom_terms']) ? (string) $_GET['woocommerce_custom_terms'] : '');
        $this->_include_post_meta 	= (isset($_GET['woocommerce_post_meta']) ? true : false);

        $this->_terms 	            = (isset($_GET['woocommerce_terms']) ? explode(',',(string) $_GET['woocommerce_terms']) : '');

        $skip_hidden 		= (int)(isset($_GET['woocommerce_fo_skip_hidden']) ? $_GET['woocommerce_fo_skip_hidden'] : 1);
        $skip_having_parent = (int)(isset($_GET['woocommerce_fo_skip_having_parent']) ? $_GET['woocommerce_fo_skip_having_parent'] : 1);

        $DELIMITER = (isset($_GET['woocommerce_fo_delimiter']) ? $_GET['woocommerce_fo_delimiter'] : WC_DELIMITER);
        if(!$DELIMITER)
        {
            $DELIMITER = '|';
        }

        if($limit && $chunk_size > $limit)
        {
            $chunk_size = $limit;
        }

        $args['post_type'] 	    = 'product';
        $args['numberposts']    = $chunk_size;
        $args['offset'] 	    = $offset;
        $args['post_status']    = 'publish';

        if(isset($_GET['woocommerce_fo_not_suppress_filters']) && @$_GET['woocommerce_fo_not_suppress_filters']=='on')
            $args['suppress_filters'] = false;

        $products  = get_posts($args);
        $_exported = 0;

        while(count($products))
        {
            foreach($products as $post)
            {
                setup_postdata($post);

                if(function_exists('wc_get_product'))
                {
                    // 3.0 compat
                    $woocommerce_product = wc_get_product($post);
                }
                else if(function_exists('get_product'))
                {
                    // 2.0 compat
                    $woocommerce_product = get_product($post);
                }
                else
                {
                    $woocommerce_product = new WC_Product($post->ID);
                }

                if(!$woocommerce_product)
                {
                    continue;
                }

                if($skip_hidden && $this->_visibility($woocommerce_product) == 'hidden')
                {
                    continue;
                }

                if($skip_having_parent && $this->_get_parent($woocommerce_product))
                {
                    continue;
                }

                $feed_item = $this->_getFeedItem($woocommerce_product);

                if ( $woocommerce_product->has_child() ) {

                    $feed_item->variants = array();

                    $children = $woocommerce_product->get_children();

                    foreach ( $children as $child ) {

                        //$child_product = $woocommerce_product->get_child( $child );
                        $child_product = $this->_get_child($woocommerce_product, $child);

                        $child_price = $child_product->get_price();

                        if ((@$feed_item->price_inc_tax == 0) && ($child_price > 0)) {

                            //$feed_item->price_ex_tax = $child_product->get_price_excluding_tax();
                            $feed_item->price_ex_tax = $this->_get_price_excluding_tax($child_product);
                            $feed_item->price_inc_tax = $child_product->get_price();

                        } else if ( ($child_price > 0) && ($child_price < @$feed_item->price_inc_tax) ) {

                            $feed_item->price_inc_tax = $child_product->get_price();
                            //$feed_item->price_ex_tax = $child_product->get_price_excluding_tax();
                            $feed_item->price_ex_tax = $this->_get_price_excluding_tax($child_product);

                        }

                        $rrp = $child_product->get_regular_price();
                        if($rrp && (!isset($feed_item->was_price) OR $feed_item->was_price>$rrp))
                        {
                            $feed_item->was_price = $rrp;
                        }

                        $variant = $this->_getFeedItem($child_product, true);
                        if($variant->is_in_stock)
                        {
                            $variant_in_stock = true;
                        }

                        $variations 		= $child_product->get_variation_attributes( );
                        $parent_attributes 	= $this->_getCustomData($woocommerce_product,'attributes');

                        $variant->_childMeta = array(
                            'variations'		=> $variations,
                            'parent_attributes'	=> $parent_attributes
                        );

                        foreach($parent_attributes as $key=>$value)
                        {
                            //sanitize_title
                            $values = array_map( 'trim', explode( $DELIMITER, $value ) );
                            $values_ = array();

                            foreach($values as $v)
                            {
                                $values_[sanitize_title($v)] = $v;
                            }

                            $parent_attributes[sanitize_title($key)] = array(
                                'name'		=> $key,
                                'values'	=> $values_
                            );

                        }

                        $_variantions = array();
                        foreach($variations as $key=>$var)
                        {
                            $_key = str_replace(array('attribute_'),'',$key);

                            if(isset($parent_attributes[$_key]) && isset($parent_attributes[$_key]['values'][$var]))
                            {
                                $_variantions[$_key] = array(
                                    'attrName'		=> $parent_attributes[$_key]['name'],
                                    'attrValue'		=> $parent_attributes[$_key]['values'][$var],
                                    'attrValueKey'	=> $var
                                );
                            }
                        }

                        //$variant->variationId = isset($child_product->variation_id) ? $child_product->variation_id : '';
                        $variation_id           = $this->_get_variation_id($child_product);
                        $variant->variationId 	= ($variation_id != '') ? $variation_id : '';
                        $variant->variations 	= $_variantions;

                        $feed_item->variants[] = $variant;
                    }

                    if(!isset($variant_in_stock))
                    {
                        continue;
                    }
                }

                //$this->feed->render_item ( $feed_item );

                if(isset($_GET['woocommerce_fo_base64'])) {
                    echo base64_encode(json_encode($feed_item)) . "\n";
                }
                else {
                    echo json_encode($feed_item) . "\n";
                }

                $_exported++;
                if($limit && $_exported>=$limit)
                {
                    exit();
                }

            }

            $args['offset'] += $chunk_size;
            $products = get_posts ( $args );

        }

        exit();
    }

    protected function _getFeedItem($woocommerce_product, $is_child = false)
    {
        $feed_item = new stdClass();

        $feed_item->pid 			= $this->_id($woocommerce_product);
        $feed_item->sku 			= $woocommerce_product->get_sku();
        $feed_item->type			= ($this->is_version_3()) ? $woocommerce_product->get_type() : $woocommerce_product->product_type ;

        if(method_exists($woocommerce_product,'get_shipping_class'))
        {
            $shipping_class_name         = $woocommerce_product->get_shipping_class();

            if($shipping_class_name)
            {
                $shipping_class     = get_term_by('slug', $shipping_class_name ,'product_shipping_class');

                if($shipping_class && is_object($shipping_class))
                {
                    $feed_item->shipping_class = [
                        'name'      => @$shipping_class->name,
                        'slug'      => @$shipping_class->slug,
                        'term_id'   => @$shipping_class->term_id
                    ];
                }
            }
        }

        $feed_item->price_ex_tax  = ($this->is_version_3()) ? wc_get_price_excluding_tax($woocommerce_product) : $woocommerce_product->get_price_excluding_tax();
        $feed_item->price_inc_tax = $woocommerce_product->get_price();
        //$feed_item->was_price   = $woocommerce_product->get_regular_price();
        $feed_item->was_price     = ($this->is_version_3()) ? $woocommerce_product->get_regular_price() : $woocommerce_product->get_price();

        // Get main item information

        $feed_item->title = get_the_title($this->_id($woocommerce_product));

        if($feed_item->type!='variation')
        {
            $feed_item->description 		= apply_filters ('the_content', get_the_content($this->_id($woocommerce_product)));

            if(get_the_excerpt($this->_id($woocommerce_product))!=get_the_content($this->_id($woocommerce_product)))
            {
                $feed_item->short_description 	= get_the_excerpt($this->_id($woocommerce_product));
            }

            $feed_item->product_url 		= get_permalink($this->_id($woocommerce_product));
        }

        //Fix for variant images
        if($is_child === true)
        {
            if(function_exists('wc_get_product_attachment_props'))
            {
                $variant_image = wc_get_product_attachment_props($woocommerce_product->get_image_id());
                $feed_item->image_url_variant_original = $variant_image['full_src'];
                $feed_item->image_url_variant_thumb    = $variant_image['thumb_src'];
            }
        }

        $feed_item->image_url_original = $this->get_the_post_thumbnail_src ( $this->_id($woocommerce_product), 'original' );
        if(!$feed_item->image_url_original) unset($feed_item->image_url_original );

        $feed_item->image_url_thumb = $this->get_the_post_thumbnail_src ( $this->_id($woocommerce_product), 'thumbnail' );
        if(!$feed_item->image_url_thumb) unset($feed_item->image_url_thumb );

        $feed_item->shipping_weight = $woocommerce_product->get_weight();

        $feed_item->is_in_stock = $woocommerce_product->is_in_stock();

        if(method_exists($woocommerce_product,'get_stock_quantity'))
        {
            $feed_item->stock_quantity = $woocommerce_product->get_stock_quantity();
        }

        $feed_item->stock 		= get_post_meta( $this->_id($woocommerce_product) ,'_stock', true );
        $feed_item->backorders 	= get_post_meta( $this->_id($woocommerce_product) ,'_backorders', true );

        $categories                 = wp_get_object_terms($this->_id($woocommerce_product), 'product_cat');
        $feed_item->categories      = array();
        $feed_item->meta_categories = array();
        foreach($categories as $c)
        {
            $bread = $this->_woo_get_term_parents($c->term_id,'product_cat',false,' > ', false, array() );

            $feed_item->categories[] = trim($bread,' >');

            $feed_item->meta_categories[] = [
                'id'        => $c->term_id,
                'name'      => $c->name,
                'slug'      => $c->slug,
                'full'      => trim($bread,' >')
            ];
        }

        if($this->_include_post_meta)
        {
            $feed_item->post_meta = get_post_meta($this->_id($woocommerce_product));
        }

        if($this->_custom_terms)
        {
            $custom_terms = wp_get_object_terms($this->_id($woocommerce_product), $this->_custom_terms);
            $feed_item->custom_terms = array();
            foreach ($custom_terms as $c) {
                $bread = $this->_woo_get_term_parents($c->term_id, $this->_custom_terms, false, ' > ', false, array());

                $feed_item->custom_terms[] = trim($bread, ' >');
            }
        }

        if($this->_terms)
        {
            $feed_item->terms = [];

            foreach($this->_terms as $term)
            {
                $_terms = wp_get_object_terms($this->_id($woocommerce_product), $term);
                $feed_item->terms[$term] = array();
                foreach ($_terms as $c) {
                    $bread = $this->_woo_get_term_parents($c->term_id, $term, false, ' > ', false, array());

                    $feed_item->terms[$term][] = trim($bread, ' >');
                }
            }
        }


        //$feed_item->categories    = implode(' > ',$feed_item->categories);

        /*
        $post = get_post($woocommerce_product->id);

        $feed_item->brand         = $this->get_product_meta($post,'mybrand');


        $feed_item->manufacturer  = $this->get_product_meta($post,'manufacturer');
        $feed_item->vendor        = $this->get_product_meta($post,'vendor');
        $feed_item->ean           = $this->get_product_meta($post,'ean');
        $feed_item->gtin          = $this->get_product_meta($post,'gtin');
        $feed_item->upc           = $this->get_product_meta($post,'upc');
        */

        $feed_item->custom_fields  		= $this->_getCustomData($woocommerce_product,'fields');
        if(isset($feed_item->custom_fields['total_sales']))
        {
            unset($feed_item->custom_fields['total_sales']);
        }

        if($feed_item->type!='variation')
        {
            $feed_item->custom_attrs  	= $this->_getCustomData($woocommerce_product,'attributes');
        }
        else
        {
            $feed_item->custom_attrs 	= $woocommerce_product->get_variation_attributes( );
        }

        // Get other images
        $feed_item->additional_images = array();

        $main_thumbnail = get_post_meta ( $this->_id($woocommerce_product), '_thumbnail_id', true );
        $images = get_children( array ( 'post_parent' => $this->_id($woocommerce_product),
            'post_status' => 'inherit',
            'post_type' => 'attachment',
            'post_mime_type' => 'image',
            'exclude' => isset($main_thumbnail) ? $main_thumbnail : '',
            'order' => 'ASC',
            'orderby' => 'menu_order' ) );



        if ( is_array ( $images ) && count ( $images ) ) {

            foreach ( $images as $image ) {

                $full_image_src = wp_get_attachment_image_src( $image->ID, 'original' );

                $feed_item->additional_images[] = $full_image_src[0];

            }

        }

        $_feed_item = new stdClass();

        foreach($feed_item as $key => $value)
        {
            if($value==="" OR (is_array($value) && !count($value))) continue;

            $_feed_item->{$key}	= $value;
        }

        return $_feed_item;
    }

    protected function _getCustomData($product,$type='fields')
    {
        $id = $this->_id($product);
        $custom = array();
        $meta = get_post_meta($id);

        if($type=='fields')
        {
            foreach($meta as $key => $value)
            {
                if(substr($key,0,1)=='_') continue;

                $custom[$key]	= count($value) === 1 ? $value[0] : $value;
                //if(!$custom[$key]) unset($custom[$key]);
            }
        }

        if($type=='attributes')
        {
            if(isset($meta['_product_attributes']))
            {
                $attributes = (unserialize($meta['_product_attributes'][0]));

                if($attributes && (is_array($attributes) OR is_object($attributes))) {
                    foreach ($attributes as $attr) {
                        $custom[$attr['name']] = $product->get_attribute($attr['name']);
                        //if(!$custom[$key]) unset($custom[$key]);
                    }
                }
            }
        }

        return $custom;
    }

    /**
     * Helper function to retrieve a custom field from a product, compatible
     * both with WC < 2.0 and WC >= 2.0
     *
     * @param WC_Product $product the product object
     * @param string $field_name the field name, without a leading underscore
     *
     * @return mixed the value of the member named $field_name, or null
     */
    /*private function get_product_meta ( $product, $field_name ) {

      if ( version_compare( WOOCOMMERCE_VERSION, "2.0.0" ) >= 0 ) {

        // even in WC >= 2.0 product variations still use the product_custom_fields array apparently
        if ( $product->variation_id && isset( $product->product_custom_fields[ '_' . $field_name ][0] ) && $product->product_custom_fields[ '_' . $field_name ][0] !== '' ) {
          return $product->product_custom_fields[ '_' . $field_name ][0];
        }


        // use magic __get
        return $product->$field_name;

      } else {

        // use product custom fields array

        // variation support: return the value if it's defined at the variation level
        if ( isset( $product->variation_id ) && $product->variation_id ) {
          if ( ( $value = get_post_meta( $product->variation_id, '_' . $field_name, true ) ) !== '' ) return $value;
          // otherwise return the value from the parent
          return get_post_meta( $product->id, '_' . $field_name, true );
        }
        // regular product
        return isset( $product->product_custom_fields[ '_' . $field_name ][0] ) ? $product->product_custom_fields[ '_' . $field_name ][0] : null;
      }

    }*/

    /**
     * Retrieve Post Thumbnail URL
     *
     * @param int     $post_id (optional) Optional. Post ID.
     * @param string  $size    (optional) Optional. Image size.  Defaults to 'post-thumbnail'.
     * @return string|bool Image src, or false if the post does not have a thumbnail.
     */
    protected function get_the_post_thumbnail_src( $post_id = null, $size = 'post-thumbnail' ) {

        $post_thumbnail_id = get_post_thumbnail_id( $post_id );

        if ( ! $post_thumbnail_id ) {
            return false;
        }

        list( $src ) = wp_get_attachment_image_src( $post_thumbnail_id, $size, false );

        return $src;
    }

    protected function _woo_get_term_parents( $id, $taxonomy, $link = false, $separator = '/', $nicename = false, $visited = array() ) {
        $chain = '';
        $parent = get_term( $id, $taxonomy );
        if ( is_wp_error( $parent ) )
            return $parent;

        if ( $nicename ) {
            $name = @$parent->slug;
        } else {
            $name = @$parent->name;
        }

        if ( @$parent->parent && ( @$parent->parent != @$parent->term_id ) && !in_array( @$parent->parent, $visited ) ) {
            $visited[] = $parent->parent;
            $chain .= $this->_woo_get_term_parents( @$parent->parent, $taxonomy, $link, $separator, $nicename, $visited );
        }

        if ( $link ) {
            $chain .= '<a href="' . get_term_link( $parent, $taxonomy ) . '" title="' . esc_attr( sprintf( __( "View all posts in %s" ), @$parent->name ) ) . '">'.@$parent->name.'</a>' . $separator;
        } else {
            $chain .= $name.$separator;
        }
        return $chain;
    } // End woo_get_term_parents()

    protected function is_version_3()
    {
        $version = '3.0.0';
        if(class_exists( 'WooCommerce'))
        {
            global $woocommerce;
            if(version_compare( $woocommerce->version, $version, ">=" ))
            {
                return true;
            }
        }
        return false;
    }

    protected function _visibility($product)
    {
        if($this->is_version_3())
        {
            return $product->get_catalog_visibility();
        }
        else
        {
            return $product->visibility;
        }

    }

    protected function _id($product)
    {
        if($this->is_version_3())
        {
            return $product->get_id();
        }
        else
        {
            return $product->id;
        }
    }

    protected function _get_parent($product)
    {
        if($this->is_version_3())
        {
            return $product->get_parent_id();
        }
        else
        {
            return $product->get_parent();
        }
    }

    protected function _get_child($product, $child_id)
    {
        if($this->is_version_3())
        {
            return wc_get_product($child_id);
        }
        else
        {
            return $product->get_child( $child_id );
        }
    }

    protected function _get_price_excluding_tax($product)
    {
        if($this->is_version_3())
        {
            if(method_exists($product,'wc_get_price_excluding_tax'))
                return $product->wc_get_price_excluding_tax();
            elseif(method_exists($product,'get_price_excluding_tax'))
                return $product->get_price_excluding_tax();
            else
                return 0;
        }
        else
        {
            return $product->get_price_excluding_tax();
        }
    }

    protected function _get_variation_id($child_product)
    {
        if($this->is_version_3())
        {
            return $child_product->get_id();
        }
        else
        {
            return $child_product->variation_id;
        }
    }

}
?>